VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "howto_tower_of_hanoi_animated"
   ClientHeight    =   3090
   ClientLeft      =   60
   ClientTop       =   450
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   3090
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Private Type Rectangle
    X As Integer
    Y As Integer
    Width As Integer
    Height As Integer
End Type

Private Const Delay As Integer = 250

' Geometry.
Private Const NumDisks As Integer = 5
Private Const Y1 As Integer = 120
Private Const Y2 As Integer = 600
Private Const DiskThickness As Integer = 240
Private X1, X2, X3, Y3 As Integer

Private PegLocation(0 To 2) As Rectangle

' Peg stacks. Each disk is represented by its width.
Private PegStack(0 To 2) As Collection

' The peg that currently holds the disks.
Private OccupiedPegNum As Integer

' True if we are performing a move.
Private Moving As Boolean

' The location of the disk that's moving.
Private MovingDiskRect As Rectangle

' Build the controls.
Private Sub Form_Load()
Dim i As Integer
Dim disk_width As Integer

    ' Make the peg stacks.
    For i = 0 To 2
        Set PegStack(i) = New Collection
    Next i

    ' Calculate the widest disk's width.
    disk_width = (NumDisks + 2) * DiskThickness

    ' See how big the form must be.
    Me.Width = 3 * disk_width + 4 * DiskThickness + (Me.Width - Me.ScaleWidth)

    ' Figure out where the pegs must be.
    X1 = CInt(DiskThickness + disk_width / 2)
    X2 = X1 + disk_width + DiskThickness
    X3 = X2 + disk_width + DiskThickness
    Y3 = ScaleHeight - 120

    PegLocation(0).X = CInt(X1 - DiskThickness / 2)
    PegLocation(0).Y = Y2
    PegLocation(0).Width = DiskThickness
    PegLocation(0).Height = Y3 - Y2

    PegLocation(1).X = CInt(X2 - DiskThickness / 2)
    PegLocation(1).Y = Y2
    PegLocation(1).Width = DiskThickness
    PegLocation(1).Height = Y3 - Y2
    
    PegLocation(2).X = CInt(X3 - DiskThickness / 2)
    PegLocation(2).Y = Y2
    PegLocation(2).Width = DiskThickness
    PegLocation(2).Height = Y3 - Y2

    ' Make the disks.
    For i = 0 To NumDisks - 1
        PegStack(0).Add disk_width
        disk_width = disk_width - DiskThickness
    Next i
    OccupiedPegNum = 0
    Moving = False
End Sub

' Move disks to this peg.
Private Sub Form_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
Dim peg_under_mouse As Integer
Dim other_peg As Integer

    If (Moving) Then Exit Sub
    peg_under_mouse = PegAtLocation(X, Y)
    If (peg_under_mouse < 0) Then Exit Sub

    Moving = True
    Me.MousePointer = vbHourglass
    DoEvents

    other_peg = 0
    If (OccupiedPegNum <> 1 And peg_under_mouse <> 1) Then other_peg = 1
    If (OccupiedPegNum <> 2 And peg_under_mouse <> 2) Then other_peg = 2

    MoveDisks OccupiedPegNum, peg_under_mouse, other_peg, NumDisks
    OccupiedPegNum = peg_under_mouse

    Me.MousePointer = vbDefault
    Moving = False
    FlushMouseEvents 0  ' Discard any pending clicks.
End Sub

' If the mouse is over a clickable peg, display a crosshair cursor.
Private Sub Form_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
Dim peg_under_mouse As Integer

    peg_under_mouse = PegAtLocation(X, Y)
    If (peg_under_mouse < 0) Then
        Me.MousePointer = vbDefault
    Else
        Me.MousePointer = vbCrosshair
    End If
End Sub

' Return the index of the peg at this location.
' Return -1 if the location is not on any peg
' or if the peg is not clickable.
Private Function PegAtLocation(ByVal X As Single, ByVal Y As Single) As Integer
Dim i As Integer

    PegAtLocation = -1
    If (Moving) Then Exit Function ' Not clickable if we're moving.

    For i = 0 To 2
        If (X >= PegLocation(i).X) And _
           (Y >= PegLocation(i).Y) And _
           (X <= PegLocation(i).X + PegLocation(i).Width) And _
           (Y <= PegLocation(i).Y + PegLocation(i).Height) _
        Then
            ' Not clickable if this is the occupied peg.
            If (i <> OccupiedPegNum) Then PegAtLocation = i
            Exit Function
        End If
    Next i
    Exit Function
End Function

' Draw the pegs and disks.
Private Sub Form_Paint()
    Me.Cls
    
    DrawPegs
    DrawDisks
End Sub

' Draw the pegs.
Private Sub DrawPegs()
Dim i As Integer

    For i = 0 To 2
        Me.Line (PegLocation(i).X, PegLocation(i).Y)-Step(PegLocation(i).Width, PegLocation(i).Height), RGB(255, 165, 0), BF
        Me.Line (PegLocation(i).X, PegLocation(i).Y)-Step(PegLocation(i).Width, PegLocation(i).Height), vbBlack, B
    Next i
End Sub

' Draw the disks.
Private Sub DrawDisks()
Dim peg As Integer
Dim Y As Integer
Dim disk_num As Integer
Dim disk_x As Integer
Dim disk_width As Integer

    ' For each peg...
    For peg = 0 To 2
        ' For each disk on the peg...
        Y = Y3 - 2 - DiskThickness
        For disk_num = 1 To PegStack(peg).Count
            ' Draw the disk.
            disk_width = PegStack(peg).Item(disk_num)
            disk_x = CInt(PegLocation(peg).X + DiskThickness / 2 - disk_width / 2)
            Me.Line (disk_x, Y)-Step(disk_width, DiskThickness), RGB(50, 205, 50), BF
            Me.Line (disk_x, Y)-Step(disk_width, DiskThickness), RGB(0, 100, 0), B
            Y = Y - (DiskThickness + 30)
        Next disk_num
    Next peg

    ' If we are currently moving a disk, draw it.
    If (Moving) Then
        Me.Line (MovingDiskRect.X, MovingDiskRect.Y)-Step(MovingDiskRect.Width, MovingDiskRect.Height), RGB(0, 191, 255), BF
        Me.Line (MovingDiskRect.X, MovingDiskRect.Y)-Step(MovingDiskRect.Width, MovingDiskRect.Height), RGB(0, 0, 255), B
    End If
End Sub

' Move num_disks disks from peg from_peg to peg to_peg.
Private Sub MoveDisks(ByVal from_peg As Integer, ByVal to_peg As Integer, ByVal other_peg As Integer, ByVal num_disks As Integer)
Dim disk_width As Integer

    ' See if we have more than one disk to move.
    If (num_disks > 1) Then
        ' We have more than one disk to move.
        ' Recursively move all but one disk to the other peg.
        MoveDisks from_peg, other_peg, to_peg, num_disks - 1
    End If

    ' Move the remaining disk.
    AnimateDiskMove from_peg, to_peg

    If (num_disks > 1) Then
        ' Recursively move the other disks back where they belong.
        MoveDisks other_peg, to_peg, from_peg, num_disks - 1
    End If
End Sub

' Animate moving a disk from peg from_peg to peg to_peg.
Private Sub AnimateDiskMove(ByVal from_peg As Integer, ByVal to_peg As Integer)
Dim disk_width As Integer
Dim Y As Integer
Dim X As Integer
Dim to_y As Integer
Dim to_x As Integer
    
    ' Remove the disk so Paint doesn't draw it.
    disk_width = PegStack(from_peg).Item(PegStack(from_peg).Count)
    PegStack(from_peg).Remove (PegStack(from_peg).Count)

    ' Move the disk above from_peg.
    Y = (PegLocation(from_peg).Y + PegLocation(from_peg).Height) - _
        (PegStack(from_peg).Count + 1) * (DiskThickness + 30)
    X = CInt(PegLocation(from_peg).X + DiskThickness / 2 - disk_width / 2)
    MovingDiskRect.X = X
    MovingDiskRect.Y = Y
    MovingDiskRect.Width = disk_width
    MovingDiskRect.Height = DiskThickness

    ' Move the disk up above the peg.
    to_y = 120
    AnimateMovement X, to_y

    ' Move the disk over the new peg.
    to_x = CInt(PegLocation(to_peg).X + DiskThickness / 2 - disk_width / 2)
    AnimateMovement to_x, to_y

    ' Move the disk down onto the peg.
    to_y = (PegLocation(to_peg).Y + PegLocation(to_peg).Height) - _
        (PegStack(to_peg).Count + 1) * (DiskThickness + 2)
    AnimateMovement to_x, to_y

    ' Redraw.
    Me.Refresh

    ' Add the disk to its new peg.
    MovingDiskRect.X = -1
    PegStack(to_peg).Add disk_width
End Sub

' Move the moving disk to this location.
Private Sub AnimateMovement(ByVal end_x As Integer, ByVal end_y As Integer)
Const twips_per_second As Integer = 6000
Dim start_x As Integer
Dim start_y As Integer
Dim dx As Single
Dim dy As Single
Dim dist As Single
Dim seconds As Single
Dim start_time As Single
Dim elapsed As Single

    start_x = MovingDiskRect.X
    start_y = MovingDiskRect.Y
    dx = end_x - MovingDiskRect.X
    dy = end_y - MovingDiskRect.Y
    dist = CSng(Sqr(dx * dx + dy * dy))

    ' Calculate distance moved per second.
    dx = twips_per_second * dx / dist
    dy = twips_per_second * dy / dist

    ' See how long the total move will take.
    seconds = dist / twips_per_second
    start_time = Timer

    ' Start moving.
    Do
        ' Redraw.
        Me.Refresh

        ' Wait a little while.
        Dim stop_time As Single
        stop_time = Timer + 0.01
        Do While Timer < stop_time
            DoEvents
        Loop

        ' See how much time has passed.
        elapsed = Timer - start_time
        If (elapsed > seconds) Then Exit Do

        ' Update the rectangle's position.
        MovingDiskRect.X = CInt(start_x + elapsed * dx)
        MovingDiskRect.Y = CInt(start_y + elapsed * dy)
    Loop

    MovingDiskRect.X = end_x
    MovingDiskRect.Y = end_y
End Sub

Private Sub Form_Unload(Cancel As Integer)
    End
End Sub
